/*
 * 1.1 Swing version.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/* Moves a foreground image in front of a background image. */

public class ClippedMovingImage extends JApplet 
                                implements ActionListener {
    AnimationPane animationPane;
    static int frameNumber = -1;
    int delay;
    static boolean frozen = false;
    Timer timer;

    Image star;
    Image rocket;
    static String rocketFile = "images/rocketship.gif";
    static String starFile = "images/starfield.gif";

    public void init() {
        //Get the images.
        star = getImage(getCodeBase(), starFile);
        rocket = getImage(getCodeBase(), rocketFile);
        buildUI(getContentPane(), star, rocket);
        startAnimation();
    }
        
    void buildUI(Container container, Image stars, Image rockets) {
        int fps = 10;

        //How many milliseconds between frames?
        delay = (fps > 0) ? (1000 / fps) : 100;

        //Set up a timer that calls this object's action handler
        timer = new Timer(delay, this);
        timer.setInitialDelay(0);
        timer.setCoalesce(true);

        animationPane = new AnimationPane(stars, rockets);
        animationPane.setBackground(Color.black);
        container.add(animationPane, BorderLayout.CENTER);

        animationPane.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if (frozen) {
                    frozen = false;
                    startAnimation();
                } else {
                    frozen = true;
                    stopAnimation();
                }
            }
       });
    }

    public void startAnimation() {
        if (frozen) { 
            //Do nothing.  The user has requested that we 
            //stop changing the image.
        } else {
            //Start animating!
            timer.start();
        }
    }

    public void stopAnimation() {
        //Stop the animating thread.
        timer.stop();
    }

    public void actionPerformed(ActionEvent e) {
        //Advance animation frame.
        frameNumber++;
        animationPane.moveIt();
    }

    class AnimationPane extends JPanel {
        Image background, foreground;
        Rectangle currentRect = new Rectangle(0,0,0,0);
        Rectangle previousRect = new Rectangle(0,0,0,0);
        Rectangle boundsRect = new Rectangle(0,0,0,0);
        int fgX = 0;
        int fgYOffset = 0; //should be configurable
    
        public AnimationPane(Image background, Image foreground) {
            this.background = background;
            this.foreground = foreground;
        }
    
        public void moveIt() {
            //Calculate the x location for the image, which can
            //be negative because we might display only the right
            //tip of the image.
            int fgWidth = foreground.getWidth(this);
            int bgWidth = background.getWidth(this);
            fgX = ((frameNumber*5) % (fgWidth + bgWidth)) - fgWidth;

            //This is too much computing.  Should be simpler...
            boundsRect.setBounds(0, 0, //XXX: could cache this
                                 bgWidth,
                                 background.getHeight(this));
            previousRect.setBounds(currentRect.x,
                                   currentRect.y,
                                   currentRect.width,
                                   currentRect.height);
            currentRect.setBounds( fgX > 0? fgX : 0, //max(x,0)
                                   fgYOffset,
                                   fgWidth,
                                   foreground.getHeight(this));
            Rectangle areaToPaint = currentRect.union(previousRect);
            areaToPaint = areaToPaint.intersection(boundsRect);
            animationPane.repaint(areaToPaint.x, areaToPaint.y, 
                                  areaToPaint.width, areaToPaint.height);
        }

        //XXX: If the component is bigger than the background,
        //XXX: then a remnant of the foreground can be left
        //XXX: hanging to the right of the background.
        //XXX: I think this occurs after a paint not triggered
        //XXX: by repaint (and thus not clipped to the background)
        //XXX: when the foreground is falling off the end of the bg.
        //Draw the current frame of animation.
        public void paintComponent(Graphics g) {
            Rectangle clipRect = g.getClipBounds();

            //Paint any area the background doesn't cover.
            g.setColor(getBackground());
            try {
                g.fillRect(clipRect.x, clipRect.y,
                           clipRect.width, clipRect.height);
            } catch (NullPointerException e) {
                System.out.println("clip bounds was null");
                g.fillRect(0, 0, getWidth(), getHeight());
            }
    
            //Draw the background image.
            g.drawImage(background, 0, 0, this);
    
            //Draw the foreground image.
            g.drawImage(foreground, fgX,
                        fgYOffset, this);
        }
    }

    public static void main(String[] args) {
        Image star = Toolkit.getDefaultToolkit().getImage(
                                ClippedMovingImage.starFile);
        Image rocket =Toolkit.getDefaultToolkit().getImage(
                                ClippedMovingImage.rocketFile);

        JFrame f = new JFrame("ClippedMovingImage");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        
        ClippedMovingImage controller = new ClippedMovingImage();
        controller.buildUI(f.getContentPane(), star, rocket);
        controller.startAnimation();

        f.setSize(new Dimension(500, 125));  
        f.setVisible(true);
    }
}
